--
-- Swarm 4 Classic
-- Version: SC 3.3.1.13
-- Author: Walkerbo
-- Email: walkerbo.saurfang@gmail.com
-- Date: Saturday 3 July 2021
--

-- I have tried to name variables, functions, frames, and buttons to be more descriptive.

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Metrics, Lists & Variables          ===========================
-- ===================================================================================================
-- ===================================================================================================

local addonName = "Swarm"
local versionNumber = "SC 3.3.1.13"

local stringClassic, stringClear, stringPause, stringTitle, stringScale, stringCamel, stringKeybindings =
    " Classic Swarm",
    "Clear List",
    " Pause Swarm",
    "SWARM",
    "Swarm Button Size",
    "Swarm",
    "Keybindings" -- default swarm strings

-- This is my first attempt at localization  **crosses fingers**
local localLanguage = GetLocale()
if localLanguage == deDE then -- German
    stringClassic = " Klassische Schwarm"
    stringClear = "Liste löschen"
    stringPause = " Pause-Schwarm"
    stringTitle = "SCHWARM"
    stringScale = "Schwarmknopfgröße"
    stringCamel = "Schwarm"
    stringKeybindings = "Tastenbelegungen"
elseif localLanguage == esES then -- Spanish (Spain)
    stringClassic = " Enjambre de clásico"
    stringClear = "Borrar lista"
    stringPause = " Enjambre de pausa"
    stringTitle = "ENJAMBRE"
    stringScale = "Tamaño del botón de enjambre"
    stringCamel = "Enjambre"
    stringKeybindings = "Atajos de teclado" -- debug --
elseif localLanguage == esMX then -- Spanish (Latin America)
    stringClassic = " Enjambre de clásico"
    stringClear = "Borrar lista"
    stringPause = " Enjambre de pausa"
    stringTitle = "ENJAMBRE"
    stringScale = "Tamaño del botón de enjambre"
    stringCamel = "Enjambre"
    stringKeybindings = "Atajos de teclado"
elseif localLanguage == frFR then -- French
    stringClassic = " Essaim classique"
    stringClear = "Effacer la liste"
    stringPause = " Essaim de Pause"
    stringTitle = "ESSAIM"
    stringScale = "Taille du bouton Swarm"
    stringCamel = "Essaim"
    stringKeybindings = "Raccourcis clavier"
elseif localLanguage == itIT then -- Italian
    stringClassic = " Sciame classico"
    stringClear = "Cancella elenco"
    stringPause = " Sciame di pausa"
    stringTitle = "SCIAME"
    stringScale = "Dimensione pulsante sciame"
    stringCamel = "Sciame"
    stringKeybindings = "Associazioni di tasti"
elseif localLanguage == koKR then -- Korean
    stringClassic = " 고전적인 떼"
    stringClear = "목록 지우기"
    stringPause = " 일시 중지 떼"
    stringTitle = "메뚜기 떼"
    stringScale = "스웜 버튼 크기"
    stringCamel = "군단"
    stringKeybindings = "키 바인딩"
elseif localLanguage == ptBR then -- Brazilian Portuguese
    stringClassic = " Enxame de clássico"
    stringClear = "Limpar lista"
    stringPause = " Enxame de pausa"
    stringTitle = "ENXAME"
    stringScale = "Tamanho do botão do enxame"
    stringCamel = "Enxame"
    stringKeybindings = "Combinações de teclas"
elseif localLanguage == ruRU then -- Russian
    stringClassic = " Классический Рой"
    stringClear = "Очистить список"
    stringPause = " Рой пауза"
    stringTitle = "РОЙ"
    stringScale = "Размер кнопки роя"
    stringCamel = "роиться"
    stringKeybindings = "Привязки клавиш"
elseif localLanguage == zhCN then -- Simplified Chinese
    stringClassic = " 经典群"
    stringClear = "清除列表"
    stringPause = " 暂停群"
    stringTitle = "群"
    stringScale = "群按钮大小"
    stringCamel = "一群"
    stringKeybindings = "键绑定"
elseif localLanguage == zhTW then -- Traditional Chinese
    stringClassic = " 經典群"
    stringClear = "清除清單"
    stringPause = " 暫停群"
    stringTitle = "群"
    stringScale = "群按鈕大小"
    stringCamel = "一群"
    stringKeybindings = "鍵綁定"
end

local ImageList = {
    bannerBlue = ("Interface\\AddOns\\Swarm\\Files\\SwarmBlueBanner"),
    bannerGreen = ("Interface\\AddOns\\Swarm\\Files\\SwarmGreenBanner"),
    buttonAddImg = ("Interface\\AddOns\\Swarm\\Files\\SwarmAdd"),
    buttonClassicIcon = ("Interface\\TargetingFrame\\UI-RaidTargetingIcon_8.png"),
    buttonDisabledIcon = ("Interface\\Worldmap\\Skull_64Grey"),
    buttonIcon = ("Interface\\Worldmap\\Skull_64Green"),
    buttonIconAlt = ("Interface\\Worldmap\\GlowSkull_64Green"),
    buttonRangeIcon = ("Interface\\Worldmap\\Skull_64Red"),
    buttonRemoveImg = ("Interface\\AddOns\\Swarm\\Files\\SwarmRemove"),
    buttonSwarmImg = ("Interface\\AddOns\\Swarm\\Files\\SwarmIcon")
}

local NumberDimensionList = {
    bannerBoxSize = 50,
    bannerGap = 30,
    bannerScale = 0.6,
    clearTargetListHeight = 45,
    cooldownTime = 30,
    edgeSize = 20,
    elementGap = 20,
    iconGap = 100,
    insetSize = 5,
    magicButtonScale = 1,
    magicButtonScaleMax = 3,
    magicButtonScaleMin = .5,
    magicButtonSize = 50,
    mainMarginGap = 25,
    scrollButtonHeight = 17,
    scrollButtonNumber = 18,
    scrollButtonWidth = 286,
    scrollFrameHeight = 321,
    scrollFrameWidth = 286,
    titleGap = -20,
    topGap = -140
}

local TextDimensionList = {
    colourBackdrop = {0, 0, 0, 1},
    colourBorderCyan = {0, 1, 1, 1},
    colourBorderGreen = {0, 1, 0, 1},
    colourTextObjects = "|cffeec900", -- gold2
    colourTextAdd = "|cff00ff00", -- green1
    colourTextNPC = "|cffff8000", -- orange
    colourTextOther = "|cff00ffff", -- cyan
    colourTextPet = "|cff7fff00", -- chartreuse1
    colourTextPlayer = "|cffb23aee", -- darkorchid2
    colourTextRemove = "|cffff0000", -- red
    sizeCheckBox = "Game20Font",
    sizeMenuButton = "SystemFont_Outline_Small",
    sizeScroll = "Game15Font",
    sizeScrollHighlight = "Game15Font_o1"
}

local swarmOnCooldownBoolean, swarmOptionsPanelVisableBoolean = false, false

-- ===========================         Global Keybinding Names          ===========================
BINDING_HEADER_SWARMTKEYBINDINGTITLE = "- Swarm -"
BINDING_NAME_SWARMTOGGLESWARMINTERFACEFRAME = "Toggle Swarm Interface Frame"
BINDING_NAME_SWARMRUNSWARMFUNCTION = "Run Swarm Function"
BINDING_NAME_SWARMADDORREMOVETARGETTOSWARMLIST = "Add/Remove Target"

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         Functions          ===========================
-- ===================================================================================================
-- ===================================================================================================

local function initialiseVariables()
    if not SwarmTargetList then
        SwarmTargetList = {}
    end
    if not SwarmOptionsList then
        SwarmOptionsList = {
            ButtonScale = 1,
            Paused = false,
            Power = true
        }
    end
end

local function updateScrollTargetList()
    FauxScrollFrame_Update(
        SwarmScrollFrame,
        #SwarmTargetList,
        NumberDimensionList.scrollButtonNumber,
        NumberDimensionList.scrollButtonHeight
    )
    for index = 1, NumberDimensionList.scrollButtonNumber do
        local offset = index + FauxScrollFrame_GetOffset(SwarmScrollFrame)
        local button = SwarmScrollFrame.buttons[index]
        button:SetNormalFontObject(TextDimensionList.sizeScroll)
        button:SetHighlightFontObject(TextDimensionList.sizeScrollHighlight)
        button.index = offset
        if offset <= #SwarmTargetList then -- get current saved target and spit into type and name
            local targetType, targetName = strsplit("-", SwarmTargetList[offset])
            if targetType == "Player" then -- colour each saved target based on type
                button:SetText(TextDimensionList.colourTextPlayer .. SwarmTargetList[offset])
            elseif targetType == "Creature" then
                button:SetText(TextDimensionList.colourTextNPC .. SwarmTargetList[offset])
            elseif targetType == "Pet" then
                button:SetText(TextDimensionList.colourTextPet .. SwarmTargetList[offset])
            else
                button:SetText(TextDimensionList.colourTextOther .. SwarmTargetList[offset])
            end
            button:Show()
        else
            button:Hide()
        end
    end
end

local function getTargetDetails()
    local targetGUID, targetName = UnitGUID("target"), UnitName("target") -- get target details
    if targetGUID then
        local targetType = strsplit("-", targetGUID) -- split target details to get type
        local currentTarget = (targetType .. "-" .. targetName) -- combine to make string
        return currentTarget
    end
end

local function testTargetOnList(currentTarget)
    for k, v in pairs(SwarmTargetList) do
        if v == currentTarget then
            return true
        end
    end
    return false
end

local function clearTargetList()
    SwarmTargetList = {}
    updateScrollTargetList()
end

local function togglePower() -- toggle the classic and current function and frames
    if SwarmOptionsList.Power then
        SwarmInterfaceFrame:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderGreen))
        SwarmKeybindingButton:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderGreen))
        SwarmInterfaceFrameBanner.Texture:SetTexture(ImageList.bannerGreen)
        SwarmPowerButton:SetChecked(false)
        SwarmScrollParent:Show()
        SwarmPauseButton:Show()
        SwarmMagicButtonScaler:Show()
        SwarmClearListButton:Show()
        if UnitExists("target") and not UnitIsDead("target") then
            if testTargetOnList(getTargetDetails()) then
                SwarmMagicButton:Show()
            end
        end
    else
        SwarmInterfaceFrame:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderCyan))
        SwarmKeybindingButton:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderCyan))
        SwarmInterfaceFrameBanner.Texture:SetTexture(ImageList.bannerBlue)
        SwarmPowerButton:SetChecked(true)
        SwarmScrollParent:Hide()
        SwarmPauseButton:Hide()
        SwarmClearListButton:Hide()
        SwarmMagicButton:Hide()
        SwarmMagicButtonScaler:Hide()
    end
end

local function togglePause() -- change text and button edge based on paused variable
    if SwarmOptionsList.Paused then
        SwarmPauseButton:SetChecked(true)
        SwarmMagicButton:Hide()
    else
        SwarmPauseButton:SetChecked(false)
        if UnitExists("target") and not UnitIsDead("target") then
            if testTargetOnList(getTargetDetails()) then
                SwarmMagicButton:Show()
            else
                SwarmMagicButton:Hide()
            end
        end
    end
end

local function swarmMenuButtonFunction(self) -- target right click swarm button
    if self.value == "Classic" then
        SWARMRUNSWARMFUNCTION()
    elseif self.value == "Add" then
        table.insert(SwarmTargetList, 1, self.arg1)
        if SwarmOptionsList.Paused then
            SwarmMagicButton:Hide()
        else
            SwarmMagicButton:Show()
        end
    elseif self.value == "Remove" then
        for k, v in pairs(SwarmTargetList) do
            if v == self.arg1 then
                table.remove(SwarmTargetList, k)
                SwarmMagicButton:Hide()
            end
        end
    end
    updateScrollTargetList()
end

local function swarmRangeCheck(self, elapsed)
    if CheckInteractDistance("target", 4) and not swarmOnCooldownBoolean then -- if swarm button visible and not on cool-down check range
        SwarmMagicButton:Enable()
    else
        SwarmMagicButton:Disable()
    end
end

-- hook the UnitPopup_ShowMenu function - this is where the menu magic happens
hooksecurefunc(
    "UnitPopup_ShowMenu",
    function()
        if (UIDROPDOWNMENU_MENU_LEVEL > 1) or UnitIsUnit("target", "player") then -- if no menu or target is the player end
            return
        end
        local info = UIDropDownMenu_CreateInfo() -- set up empty info variable
        info.text = stringTitle
        info.owner = which
        info.notCheckable = 1
        info.fontObject = TextDimensionList.sizeMenuButton
        info.arg1 = getTargetDetails()
        info.func = swarmMenuButtonFunction
        if not SwarmOptionsList.Power then -- if swarm classic make button blue
            info.colorCode = TextDimensionList.colourTextOther
            info.icon = ImageList.buttonClassicIcon
            info.value = "Classic"
        elseif not testTargetOnList(getTargetDetails()) then -- if target on list make button green
            info.colorCode = TextDimensionList.colourTextAdd
            info.icon = ImageList.buttonAddImg
            info.value = "Add"
        else -- else make button red
            info.colorCode = TextDimensionList.colourTextRemove
            info.icon = ImageList.buttonRemoveImg
            info.value = "Remove"
        end
        UIDropDownMenu_AddSeparator(UIDROPDOWNMENU_MENU_LEVEL) -- add a menu separator
        UIDropDownMenu_AddButton(info) -- add button
        UIDropDownMenu_AddSeparator(UIDROPDOWNMENU_MENU_LEVEL) -- add a menu separator
    end
)

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         global functions for keybinds       ===========================
-- ===================================================================================================
-- ===================================================================================================

-- ===========================         swarm function          ===========================
function SWARMRUNSWARMFUNCTION()
    local chatString, playerXLoc, playerYLoc
    if UnitIsDead("target") then
        SetRaidTarget("target", 0) -- Clear any raid target markers
        chatString = (stringCamel .. ":: " .. (UnitName("target")) .. " - is dead") -- target is dead
        SendChatMessage(chatString, "CHANNEL", nil, 1) -- Sent to general chat
    else
        SetRaidTarget("target", 0) -- Clear any raid target markers
        SetRaidTarget("target", 8) -- Apply skull raid target marker
        chatString =
            (stringCamel ..
            ":: " ..
                (UnitName("target")) ..
                    " - " .. (math.floor(UnitHealth("target") / UnitHealthMax("target") * 100)) .. "%" .. " health") -- Get target name and health
        SendChatMessage(chatString, "CHANNEL", nil, 1) -- Sent to general chat
        playerXLoc, playerYLoc = C_Map.GetPlayerMapPosition(C_Map.GetBestMapForUnit("player"), "player"):GetXY() -- Get player location
        chatString =
            (stringCamel .. ":: " .. (format("%s - %.1f, %.1f", GetSubZoneText(), playerXLoc * 100, playerYLoc * 100))) -- Over write previous string with the player location
        SendChatMessage(chatString, "CHANNEL", nil, 1) -- Send to chat
    end
end

-- ===========================         add or remove target          ===========================
function SWARMADDORREMOVETARGETTOSWARMLIST()
    if UnitExists("target") then
        currentTarget = getTargetDetails()
        if testTargetOnList(currentTarget) then
            for k, v in pairs(SwarmTargetList) do
                if v == currentTarget then
                    table.remove(SwarmTargetList, k)
                    SwarmMagicButton:Hide()
                end
            end
        else
            table.insert(SwarmTargetList, 1, currentTarget)
            if SwarmOptionsList.Power and not SwarmOptionsList.Paused then
                SwarmMagicButton:Show()
            end
        end
    end
    updateScrollTargetList()
end

-- ===========================         toggle swarm frame          ===========================
function SWARMTOGGLESWARMINTERFACEFRAME()
    if InterfaceOptionsFrame:IsShown() then
        InterfaceOptionsFrame:Hide()
    else
        InterfaceOptionsFrame_OpenToCategory(SwarmInterfaceFrame)
        InterfaceOptionsFrame_OpenToCategory(SwarmInterfaceFrame)
    end
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================         slash commands          ===========================
-- ===================================================================================================
-- ===================================================================================================

-- ===========================         toggle swarm frame          ===========================

SLASH_SWARM1 = "/swarm"
function SlashCmdList.SWARM(msg, editbox)
    SWARMTOGGLESWARMINTERFACEFRAME()
end

-- ===========================         reset button size and position          ===========================
SLASH_SWARMP1 = "/swarmpos" -- Reposition swarm magic button
function SlashCmdList.SWARMP(msg, editbox)
    SwarmMagicButton:ClearAllPoints()
    SwarmMagicButton:SetPoint("CENTER")
    SwarmOptionsList.ButtonScale = 1
    SwarmMagicButton:SetScale(SwarmOptionsList.ButtonScale)
    SwarmMagicButton:Show()
end

-- ===================================================================================================
-- ===================================================================================================
-- ===========================                 Frames                      ===========================
-- ===================================================================================================
-- ===================================================================================================

local SwarmInterfaceFrame = CreateFrame("Frame", "SwarmInterfaceFrame", UIParent)
SwarmInterfaceFrame.name = "Swarm"
SwarmInterfaceFrame:SetBackdrop(
    {
        bgFile = "Interface\\Buttons\\WHITE8X8",
        insets = {
            left = NumberDimensionList.insetSize,
            right = NumberDimensionList.insetSize,
            top = NumberDimensionList.insetSize,
            bottom = NumberDimensionList.insetSize
        },
        tileSize = nil,
        tile = false,
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        edgeSize = NumberDimensionList.edgeSize
    }
)
SwarmInterfaceFrame:SetBackdropColor(unpack(TextDimensionList.colourBackdrop))
SwarmInterfaceFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        if event == "PLAYER_LOGIN" then -- fires when a player logs in or reloads
            print(TextDimensionList.colourTextNPC .. " *** " .. addonName .. " - " .. versionNumber .. " Loaded *** \n") -- Load Message
            swarmSelfPlayerGUID = UnitGUID("player")
            initialiseVariables()
            updateScrollTargetList()
            togglePause()
            togglePower()
            SwarmMagicButton:Hide()
        elseif event == "PLAYER_ENTERING_WORLD" then -- fires when the player sees a loading screen
            SwarmMagicButton:Hide()
        elseif event == "PLAYER_TARGET_CHANGED" then -- fires when player target changes
            SwarmMagicButton:Hide()
            if UnitExists("target") and not UnitIsDead("target") then -- must have an alive target
                if UnitIsUnit("target", "player") and SwarmOptionsList.Power then -- if the player is the target the button displays
                    SwarmMagicButton:Show()
                    SwarmMagicButton:SetMovable(true)
                elseif testTargetOnList(getTargetDetails()) and not SwarmOptionsList.Paused and SwarmOptionsList.Power then
                    SwarmMagicButton:Show()
                    SwarmMagicButton:SetMovable(false)
                    SwarmMagicButton:Disable()
                    if CheckInteractDistance("target", 4) then
                        SwarmMagicButton:Enable()
                    end
                end
            end
        else
            SwarmMagicButton:Hide()
        end
        SwarmMagicButton:SetScale(SwarmOptionsList.ButtonScale)
    end
)
SwarmInterfaceFrame:RegisterEvent("PLAYER_LOGIN") -- fires when a player logs in or reloads
SwarmInterfaceFrame:RegisterEvent("PLAYER_ENTERING_WORLD") -- fires when the player sees a loading screen
SwarmInterfaceFrame:RegisterEvent("PLAYER_TARGET_CHANGED") -- fires when the player target changes
InterfaceOptions_AddCategory(SwarmInterfaceFrame)

local SwarmInterfaceFrameBanner = CreateFrame("Frame", "SwarmInterfaceFrameBanner", SwarmInterfaceFrame)
SwarmInterfaceFrameBanner:SetSize(NumberDimensionList.bannerBoxSize, NumberDimensionList.bannerBoxSize)
SwarmInterfaceFrameBanner:SetScale(NumberDimensionList.bannerScale)
SwarmInterfaceFrameBanner:SetPoint("TOP", 0, 0)
SwarmInterfaceFrameBanner.Texture = SwarmInterfaceFrameBanner:CreateTexture(nil, "BACKGROUND")
SwarmInterfaceFrameBanner.Texture:SetPoint("TOP", 0, NumberDimensionList.bannerGap)

local SwarmScrollParent =
    CreateFrame("Frame", "SwarmScrollParent", SwarmInterfaceFrame, "TooltipBorderedFrameTemplate")
SwarmScrollParent:SetBackdrop(
    {
        bgFile = nil,
        insets = nil,
        tileSize = nil,
        tile = false,
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        edgeSize = NumberDimensionList.edgeSize
    }
)
SwarmScrollParent:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderGreen))
SwarmScrollParent:SetSize(NumberDimensionList.scrollFrameWidth, NumberDimensionList.scrollFrameHeight)
SwarmScrollParent:SetPoint("TOPLEFT", NumberDimensionList.mainMarginGap, NumberDimensionList.topGap)

local SwarmScrollFrame = CreateFrame("ScrollFrame", "SwarmScrollFrame", SwarmScrollParent, "FauxScrollFrameTemplate")
SwarmScrollFrame:SetPoint("TOPLEFT", 0, -8)
SwarmScrollFrame:SetPoint("BOTTOMRIGHT", -30, 8)
SwarmScrollFrame:SetScript(
    "OnVerticalScroll",
    function(self, offset)
        FauxScrollFrame_OnVerticalScroll(self, offset, NumberDimensionList.scrollButtonNumber, updateScrollTargetList)
    end
)

SwarmScrollFrame.buttons = {}
for index = 1, NumberDimensionList.scrollButtonNumber do
    SwarmScrollFrame.buttons[index] =
        CreateFrame("Button", "stb" .. index, SwarmScrollParent, "OptionsListButtonTemplate")
    local button = SwarmScrollFrame.buttons[index]
    button:SetSize(NumberDimensionList.scrollButtonWidth, NumberDimensionList.scrollButtonHeight)
    button:SetPoint("TOPLEFT", 8, -(index - 1) * NumberDimensionList.scrollButtonHeight - 8)
    button:SetScript(
        "OnClick",
        function(self)
            table.remove(SwarmTargetList, self.index)
            updateScrollTargetList()
        end
    )
end

local SwarmPowerButton = CreateFrame("CheckButton", "SwarmPowerButton", SwarmInterfaceFrame, "UICheckButtonTemplate")
SwarmPowerButton:SetPoint("TOPLEFT", SwarmScrollParent, "TOPRIGHT", NumberDimensionList.elementGap, 0)
SwarmPowerButton.text:SetFontObject(TextDimensionList.sizeCheckBox)
SwarmPowerButton.text:SetText(stringClassic)
SwarmPowerButton:SetScript(
    "OnClick",
    function()
        if SwarmOptionsList.Power then
            SwarmOptionsList.Power = false
        else
            SwarmOptionsList.Power = true
        end
        togglePower()
    end
)

local SwarmPauseButton = CreateFrame("CheckButton", "SwarmPauseButton", SwarmInterfaceFrame, "UICheckButtonTemplate")
SwarmPauseButton:SetPoint("TOP", SwarmPowerButton, "BOTTOM", 0, NumberDimensionList.titleGap)
SwarmPauseButton.text:SetFontObject(TextDimensionList.sizeCheckBox)
SwarmPauseButton.text:SetText(stringPause)
SwarmPauseButton:SetScript(
    "OnClick",
    function()
        if SwarmOptionsList.Paused then
            SwarmOptionsList.Paused = false
        else
            SwarmOptionsList.Paused = true
        end
        togglePause()
    end
)

local SwarmClearListButton = CreateFrame("Button", "SwarmClearListButton", SwarmInterfaceFrame, "UIMenuButtonStretchTemplate")
SwarmClearListButton:SetSize(NumberDimensionList.scrollFrameWidth, NumberDimensionList.clearTargetListHeight)
SwarmClearListButton:SetPoint("TOP", SwarmScrollParent, "BOTTOM", 0, NumberDimensionList.titleGap)
SwarmClearListButton:SetBackdrop(
    {
        bgFile = nil,
        insets = nil,
        tileSize = nil,
        tile = true,
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        edgeSize = 20
    }
)
SwarmClearListButton:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderGreen))
SwarmClearListButton:SetNormalFontObject(TextDimensionList.sizeCheckBox)
SwarmClearListButton:SetHighlightFontObject(TextDimensionList.sizeCheckBox)
SwarmClearListButton:SetText(TextDimensionList.colourTextObjects .. stringClear)
SwarmClearListButton:RegisterForClicks("RightButtonUp")
SwarmClearListButton:SetScript(
    "OnClick",
    function()
        clearTargetList()
    end
)

local SwarmInterfaceFrameIcon = CreateFrame("Frame", "SwarmInterfaceFrameIcon", SwarmInterfaceFrame)
SwarmInterfaceFrameIcon:SetSize(NumberDimensionList.scrollFrameWidth, NumberDimensionList.bannerBoxSize)
SwarmInterfaceFrameIcon:SetScale(NumberDimensionList.bannerScale)
SwarmInterfaceFrameIcon:SetPoint("BOTTOMLEFT", SwarmScrollParent, "BOTTOMRIGHT", NumberDimensionList.iconGap, 0)
SwarmInterfaceFrameIcon.Texture = SwarmInterfaceFrameIcon:CreateTexture(nil, "BACKGROUND")
SwarmInterfaceFrameIcon.Texture:SetPoint("BOTTOM", 0, 0)
SwarmInterfaceFrameIcon.Texture:SetTexture(ImageList.buttonSwarmImg)



local SwarmKeybindingButton = CreateFrame("Button", "SwarmKeybindingButton", SwarmInterfaceFrame, "UIMenuButtonStretchTemplate")
SwarmKeybindingButton:SetSize(NumberDimensionList.scrollFrameWidth, NumberDimensionList.clearTargetListHeight)
SwarmKeybindingButton:SetPoint("TOP", SwarmInterfaceFrameIcon, "BOTTOM", 0, NumberDimensionList.titleGap)
SwarmKeybindingButton:SetBackdrop(
    {
        bgFile = nil,
        insets = nil,
        tileSize = nil,
        tile = true,
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        edgeSize = 20
    }
)
SwarmKeybindingButton:SetBackdropBorderColor(unpack(TextDimensionList.colourBorderGreen))
SwarmKeybindingButton:SetNormalFontObject(TextDimensionList.sizeCheckBox)
SwarmKeybindingButton:SetHighlightFontObject(TextDimensionList.sizeCheckBox)
SwarmKeybindingButton:SetText(TextDimensionList.colourTextObjects .. stringKeybindings)
SwarmKeybindingButton:SetScript(
    "OnClick",
    function()
        GameMenuButtonKeybindings:Click("RightButton")
KeyBindingFrame:Show()
for _, v in pairs(KeyBindingFrame.categoryList.buttons) do
    if v:GetText() == "Walkerbo's Addons" or v:GetText() == "WALKERBO'S ADDONS" then
        v:Click()
    end
end
    end
)






local SwarmMagicButtonScaler =
    CreateFrame("Slider", "SwarmMagicButtonScaler", SwarmInterfaceFrame, "OptionsSliderTemplate")
SwarmMagicButtonScaler:SetPoint("TOPLEFT", SwarmPauseButton, "BOTTOMRIGHT", 0, NumberDimensionList.titleGap * 2.5)
SwarmMagicButtonScaler:SetScale(NumberDimensionList.magicButtonScale)
SwarmMagicButtonScaler.textLow = _G["SwarmMagicButtonScaler" .. "Low"]
SwarmMagicButtonScaler.textHigh = _G["SwarmMagicButtonScaler" .. "High"]
SwarmMagicButtonScaler.text = _G["SwarmMagicButtonScaler" .. "Text"]
SwarmMagicButtonScaler:SetMinMaxValues(NumberDimensionList.magicButtonScaleMin, NumberDimensionList.magicButtonScaleMax)
SwarmMagicButtonScaler.minValue, SwarmMagicButtonScaler.maxValue = SwarmMagicButtonScaler:GetMinMaxValues()
SwarmMagicButtonScaler.textLow:SetText((NumberDimensionList.magicButtonScaleMin * 100) .. ("%"))
SwarmMagicButtonScaler.textHigh:SetText((NumberDimensionList.magicButtonScaleMax * 100) .. ("%"))
SwarmMagicButtonScaler.text:SetFontObject(TextDimensionList.sizeCheckBox)
SwarmMagicButtonScaler.text:SetText(stringPause)
SwarmMagicButtonScalerText:SetText(TextDimensionList.colourTextObjects .. stringScale)
SwarmMagicButtonScaler:SetValue(.5)
SwarmMagicButtonScaler:SetValueStep(1)
SwarmMagicButtonScaler:SetScript(
    "OnValueChanged",
    function(self, event, arg1)
        SwarmOptionsList.ButtonScale = tonumber(event)
        SwarmMagicButton:SetScale(SwarmOptionsList.ButtonScale)
    end
)

local SwarmMagicButton = CreateFrame("Button", "SwarmMagicButton", UIParent)
SwarmMagicButton:SetSize(NumberDimensionList.magicButtonSize, NumberDimensionList.magicButtonSize)
SwarmMagicButton:ClearAllPoints()
SwarmMagicButton:SetPoint("CENTER")
SwarmMagicButton:SetNormalTexture(ImageList.buttonIcon)
SwarmMagicButton:SetHighlightTexture(ImageList.buttonIconAlt, "BLEND")
SwarmMagicButton:SetDisabledTexture(ImageList.buttonRangeIcon)
SwarmMagicButton:SetMovable(true)
SwarmMagicButton:SetClampedToScreen(true)
SwarmMagicButton:RegisterForDrag("RightButton")
SwarmMagicButton:SetScript("OnDragStart", SwarmMagicButton.StartMoving)
SwarmMagicButton:SetScript("OnDragStop", SwarmMagicButton.StopMovingOrSizing)
SwarmMagicButton:SetScript("OnUpdate", swarmRangeCheck)
SwarmMagicButton:SetToplevel(true)

local SwarmMagicButtonCooldown =
    CreateFrame("Cooldown", "SwarmMagicButtonCooldown", SwarmMagicButton, "CooldownFrameTemplate")
SwarmMagicButtonCooldown:SetSwipeTexture(ImageList.buttonIcon)
SwarmMagicButtonCooldown:SetEdgeTexture(ImageList.buttonDisabledIcon)
SwarmMagicButtonCooldown:SetDrawEdge(false)
SwarmMagicButton:SetScript(
    "OnClick",
    function(self)
        if UnitIsUnit("target", "player") then -- if the target is the player toggle the swarm options panel
            openSwarmInterfaceFrame()
        else
            SWARMRUNSWARMFUNCTION()
            swarmOnCooldownBoolean = true -- change cool-down boolean to true and switch to the disabled texture
            SwarmMagicButton:SetDisabledTexture(ImageList.buttonDisabledIcon)
            SwarmMagicButton:Disable() -- disable button and run cool-down timer
            SwarmMagicButtonCooldown:SetCooldown(GetTime(), NumberDimensionList.cooldownTime)
            C_Timer.NewTimer(
                NumberDimensionList.cooldownTime,
                function(self)
                    swarmOnCooldownBoolean = false -- change cool-down boolean to false and switch to the out of range texture
                    SwarmMagicButton:SetDisabledTexture(ImageList.buttonRangeIcon)
                    SwarmMagicButton:Enable() -- enable button after cool-down completed
                end
            )
        end
    end
)
